#
#  VirtualMemory.py
#  HaloDemoMemoryTemplate
#
#  Created by nil on 12/14/08.
#  Copyright (c) 2009. All rights reserved.
#

# global variable that is guaranteed to be modified during execution
CURRENT_ENDIAN_BYTE_ORDER = -1

# Constants that shouldn't be changed, ever
NATIVE_ENDIAN_BYTE_ORDER =  0
BIG_ENDIAN_BYTE_ORDER =     1
LITTLE_ENDIAN_BYTE_ORDER =  2

# User configurable constants
# (Changes to these constants only take effect after restarting this memory template application)

# Set this to whatever application to target (e.g, "Halo" for Halo full)
APP_TARGET =                "Halo Demo"

# Set this to the default byte order, which can be NATIVE_ENDIAN_BYTE_ORDER, BIG_ENDIAN_BYTE_ORDER, or LITTLE_ENDIAN_BYTE_ORDER
# (one would most likely want BIG_ENDIAN_BYTE_ORDER for rosetta-emulated apps, and NATIVE_ENDIAN_BYTE_ORDER for any other type of app)
DEFAULT_ENDIAN_BYTE_ORDER = BIG_ENDIAN_BYTE_ORDER

# --- end ---

from Foundation import *
from AppKit import *
from objc import *

PROCESS_IDENTIFIER = None

def loadProcess():
	for applicationDictionary in NSWorkspace.sharedWorkspace().launchedApplications():
		if applicationDictionary.objectForKey_("NSApplicationName") == APP_TARGET:
			global PROCESS_IDENTIFIER
			PROCESS_IDENTIFIER = applicationDictionary.objectForKey_("NSApplicationProcessIdentifier").intValue()
			
#Can return either NATIVE_ENDIAN_BYTE_ORDER, BIG_ENDIAN_BYTE_ORDER, or LITTLE_ENDIAN_BYTE_ORDER
def getEndianByteOrder():
	global CURRENT_ENDIAN_BYTE_ORDER
	return CURRENT_ENDIAN_BYTE_ORDER
		
#Can set either NATIVE_ENDIAN_BYTE_ORDER, BIG_ENDIAN_BYTE_ORDER, or LITTLE_ENDIAN_BYTE_ORDER
def setEndianByteOrder(newByteOrder):
	global CURRENT_ENDIAN_BYTE_ORDER
	CURRENT_ENDIAN_BYTE_ORDER = newByteOrder
	
def readInt8(memoryAddress):
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readInt8AtProcess_memoryAddress_(PROCESS_IDENTIFIER, memoryAddress)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.charValue()
	
	return None

def writeInt8(memoryAddress, value):
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeInt8AtProcess_memoryAddress_value_(PROCESS_IDENTIFIER, memoryAddress, value) == YES
	
def readUInt8(memoryAddress):
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readUnsignedInt8AtProcess_memoryAddress_(PROCESS_IDENTIFIER, memoryAddress)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.unsignedCharValue()
	
	return None
	
def writeUInt8(memoryAddress, value):
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeUnsignedInt8AtProcess_memoryAddress_value_(PROCESS_IDENTIFIER, memoryAddress, value) == YES
	
def readInt16(memoryAddress, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readInt16AtProcess_memoryAddress_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, byteOrder)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.shortValue()
	
	return None
	
def writeInt16(memoryAddress, value, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeInt16AtProcess_memoryAddress_value_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, value, byteOrder) == YES
	
def readUInt16(memoryAddress, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readUnsignedInt16AtProcess_memoryAddress_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, byteOrder)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.unsignedShortValue()
	
	return None
	
def writeUInt16(memoryAddress, value, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeUnsignedInt16AtProcess_memoryAddress_value_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, value, byteOrder) == YES

def readInt32(memoryAddress, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readInt32AtProcess_memoryAddress_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, byteOrder)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.longValue()	
	
	return None
	
def writeInt32(memoryAddress, value, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeInt32AtProcess_memoryAddress_value_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, value, byteOrder) == YES

def readUInt32(memoryAddress, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readUnsignedInt32AtProcess_memoryAddress_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, byteOrder)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.unsignedLongValue()
		
	return None
	
def writeUInt32(memoryAddress, value, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeUnsignedInt32AtProcess_memoryAddress_value_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, value, byteOrder) == YES

def readInt64(memoryAddress, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readInt64AtProcess_memoryAddress_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, byteOrder)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.longLongValue()
	
	return None

def writeInt64(memoryAddress, value, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeInt64AtProcess_memoryAddress_value_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, value, byteOrder) == YES

def readUInt64(memoryAddress, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readUnsignedInt64AtProcess_memoryAddress_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, byteOrder)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.unsignedLongLongValue()
	
	return None

def writeUInt64(memoryAddress, value, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeUnsignedInt64AtProcess_memoryAddress_value_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, value, byteOrder) == YES

def readFloat(memoryAddress, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readFloatAtProcess_memoryAddress_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, byteOrder)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.floatValue()
		
	return None
	
def writeFloat(memoryAddress, value, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeFloatAtProcess_memoryAddress_value_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, value, byteOrder) == YES
  
def readDouble(memoryAddress, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	global PROCESS_IDENTIFIER
	numberObject = HDTVirtualMemory.readDoubleAtProcess_memoryAddress_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, byteOrder)
	
	if isinstance(numberObject, NSNumber):
		return numberObject.doubleValue()
		
	return None
	
def writeDouble(memoryAddress, value, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeDoubleAtProcess_memoryAddress_value_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, value, byteOrder) == YES

def readUTF8String(memoryAddress):
	global PROCESS_IDENTIFIER
	stringObject = HDTVirtualMemory.readUTF8StringAtProcess_memoryAddress_(PROCESS_IDENTIFIER, memoryAddress)
	#get rid of the extra last 0 terminator character
	return stringObject[0:-1]
	
def writeUTF8String(memoryAddress, value):
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeUTF8StringAtProcess_memoryAddress_value_(PROCESS_IDENTIFIER, memoryAddress, value) == YES
	
def readUTF16String(memoryAddress, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	stringObject = HDTVirtualMemory.readUTF16StringAtProcess_memoryAddress_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, byteOrder)
	#get rid of the extra last 0 terminator character
	return stringObject[0:-1]
	
def writeUTF16String(memoryAddress, value, byteOrder=None):
	if byteOrder == None:
		byteOrder = CURRENT_ENDIAN_BYTE_ORDER
	
	global PROCESS_IDENTIFIER
	return HDTVirtualMemory.writeUTF16StringAtProcess_memoryAddress_value_byteOrder_(PROCESS_IDENTIFIER, memoryAddress, value, byteOrder) == YES
